/*
 ADPCM.c knihovna pro praci s ADPCM kodovanim zvuku
*/

const signed char IndexTable[16] = {-1,-1,-1,-1,2,4,6,8,-1,-1,-1,-1,2,4,6,8};

// Quantizer step size lookup table
const long StepSizeTable[89] =
      {7,8,9,10,11,12,13,14,16,17,19,21,23,25,28,31,34,37,41,
       45,50,55,60,66,73,80,88,97,107,118,130,143,157,173,190,
       209,230,253,279,307,337,371,408,449,494,544,598,658,724,
       796,876,963,1060,1166,1282,1411,1552,1707,1878,2066,2272,
       2499,2749,3024,3327,3660,4026,4428,4871,5358,5894,6484,
       7132,7845,8630,9493,10442,11487,12635,13899,15289,16818,
       18500,20350,22385,24623,27086,29794,32767};

static signed long diff;                        // Diff. between sample and predicted sample
static signed int step;                               // Quantizer step size
static signed long predsample,diffq,prevsample=0;            // ADPCM predictor output, Predicted diff.
static signed int eindex,previndex=0;                             // Index into step size table

void ADPCMInit() {
 prevsample=0;
 previndex=50;
}

signed long int ADPCMDecoder( unsigned char code ){
  predsample = prevsample;                      // Restore previous values
  eindex      = previndex;
  code=code&0x0F;
  //fprintf(seriak,"%5d %5d\n",eindex,code);
  step = StepSizeTable[eindex];                        // Find quantizer step size
  diffq = step >> 3;                                  // Inverse quantize ADPCM code into a
  if( code & 4 ) diffq += step;                       // diff. using the quantizer step
  if( code & 2 ) diffq += step >> 1;
  if( code & 1 ) diffq += step >> 2;
  if( code & 8 ) predsample -= diffq;                 // Add the difference to predicted sample
  else           predsample += diffq;

  if( predsample > 32767 )       predsample =  32767; // Check if overflow of new predicted sample
  else if( predsample < -32768 ) predsample = -32768;
  eindex += IndexTable[code];                          // Find new quantizer step size
  if( eindex == 0  ) eindex = 1;                         // Check if overflow of new quantizer step
  if( eindex > 88 ) eindex = 88;
  //fprintf(seriak,"%3d %3d\n",IndexTable[code],eindex);
  prevsample = predsample;                      // Save values for next iteration
  previndex  = eindex;
  return(predsample);                               // Return new sample
  
}

char ADPCMEncoder( signed short sample ){
  unsigned char code;                            // ADPCM output value
  signed int cindex;
  predsample = prevsample;        // Restore previous values of predicted
  cindex = previndex;              // sample and quantizer step size index
  step = StepSizeTable[cindex];

  diff = sample - predsample;           // Compute diff. between actual sample
  if((signed long)diff >= 0)                         // and the predicted sample
    code = 0;
  else {
    code = 8;
    diff = -diff;
  }
  diffq = step >> 3;                    // Quantize the diff. into 4-bit ADPCM code
  if(diff >= step ){                   // using the quantizer step size
    code |= 4;                          // Inverse quantize the ADPCM code into a
    diff -= step;                       // predicted diff. using the quantizer step
    diffq += step;                      // size
  }
  step >>= 1;
  if( diff >= step ){
    code |= 2;
    diff -= step;
    diffq += step;
  }
  step >>= 1;
  if( diff >= step ){
    code |= 1;
    diffq += step;
  }
  if( code & 8 )                        // Compute new predicted sample by adding
    predsample -= diffq;                // the old predicted sample to new diff.
  else
    predsample += diffq;
  if( predsample > 32767 )              // Check if overflow of the new pred. sample
    predsample = 32767;
  else if( predsample < -32768 )
    predsample = -32768;

  cindex += IndexTable[code];            // Find new quantizer stepsize
  //fprintf(seriak,"%5d %5d\n",cindex,code);
  
  if((signed int) cindex < 0  ) cindex = 0;           // Check if overflow of new quantizer step
  if( cindex > 88 ) cindex = 88;
  
  prevsample = predsample;        // Save values for next iteration
  previndex = cindex;
  return ( code & 0x0f );               // Return the ADPCM code
}


//unsigned char isdata=0,cnt=0,cnt2=0,msb=0,hodnota;
/*
SIGNAL(SIG_OUTPUT_COMPARE1A) {
 //unsigned char inp,decoderout;
 signed long decoderout;
 cli();
 TCNT1=0; 
 if (isdata) {
  //dame data na port
  PORTA&=~1;
  PORTD|=64;
  if (msb) {
   hodnota=PlayBuffer[cnt]>>4;
   cnt++;
   if (cnt==128) {
    isdata=0;
    cnt=0;
   }
   if (cnt==10) {
    uart_transmit('N');
   } 
  } else {
   hodnota=PlayBuffer[cnt]&0x0F;
  }
  msb^=0xff;
  PORTD&=~64;
  decoderout=((signed long)ADPCMDecoder(hodnota)+32768)/256;
  
  PORTC=(unsigned char) decoderout;
  
  cnt2=200;
 } else {
  //data nejsou
  PORTA|=1;
  cnt2--;
  if (!cnt2) {uart_transmit('N');cnt2=250;}
 }
//decoderout=((ADPCMDecoder(inp) + 32768) / 256);
 sei();
}
*/
