/*
##############################################################
   Rozpoznavani hlasu
##############################################################
   
   (c) Ivo Strasil 2004
   Wurmova 22
   602 00 Brno
   ivo.strasil@centrum.cz   
*/
#define VERZE		"1.0"
/*
   Licence GPL - Open Source
##############################################################
*/



//includuj standardni knihovny

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <stdio.h>
#include <ctype.h>

//definice seriaku
FILE * seriak;

//je v 1, pokud je pomoci RS232 predan znak Esc
unsigned char interruptflag=0;

#define LEDBUSY		1
#define VRBUTTON	2
#define PWRSAVE		4
#define LEDSPEAK	8

unsigned char conf=LEDBUSY+VRBUTTON+PWRSAVE+LEDSPEAK;


//includuj knihovny projektu
#include "a2d.c"		//ad prevodnik
#include "defs.c"		//definice systemu - SPI, UART, ...
#include "adpcm.c"		//ADPCM koder
#include "recorder.c"	//nahravani a prehravani zvuku
#include "recog.c"	    //rozpoznavani reci
#include "cisla8b.c"    //tabulka pozic v cisla8b.wav a 
                        //procedury pro obsluhu vystupu cisel

unsigned char CmdBuf[20];
unsigned char CmdPos=0;
unsigned char CmdFlag=0;

SIGNAL (SIG_UART_RECV) {
 unsigned char data=UDR;

 if (CmdFlag&&data==27) {
  interruptflag=1;
 }
 if (data==13||data==10||CmdFlag) {
  CmdFlag=1;
 } else {
  if (CmdPos<20&&data!=27) { //ignoruj prebytecne [ESC]
   CmdBuf[CmdPos]=data; 
   CmdPos++;
   interruptflag=0;
  } 
 } 
}

void CommandDone() {
 for (unsigned char i=0;i<20;i++) {
  CmdBuf[i]=0x00;
 }
 CmdPos=0;
 CmdFlag=0;
 interruptflag=0;
}

unsigned short from,length;

void DecodeLF()
{
      unsigned char k=3;
	  from=0;
	  while (isdigit(CmdBuf[k])&&k<18) {
	   from*=10;
	   from+=CmdBuf[k]-'0';
	   k++;
	  }
	  k++;
	  length=0;
	  while (isdigit(CmdBuf[k])&&k<19) {
	   length*=10;
	   length+=CmdBuf[k]-'0';
	   k++;
	  }
};

void loading(unsigned short from, unsigned short length)
{   

    // interrupt disabled, SPI port enabled, master mode, MSB first, SPI mode 3, Fcl/4
    SPCR = 0x5C;           
   
    
	TCNT1=0;
	unsigned char datatemp,msb;
	buffer_counter=0;
	page_counter=from;
	length+=from;
	//vypni preruseni
	cli();
    do
    {
		datatemp=ReceiveByte();
		if (datatemp==27) {
		 interruptflag=1;
		} else 
		 if (datatemp<='F'&&datatemp>='A') 
		  {datatemp=10+datatemp-'A';} 
		   else
		  {datatemp-='0';}
		msb=datatemp<<4;
		if (!interruptflag) {
		 datatemp=ReceiveByte();
		 if (datatemp==27) {
		  interruptflag=1;
		 } else 
		  if (datatemp<='F'&&datatemp>='A') 
		   {datatemp=10+datatemp-'A';} 
		    else
		   {datatemp-='0';}
		}   		
		
		PORTA|=128;
		//nacti vyssi nibble
		
		//nizsi nibble
		if (!interruptflag) {write_to_flash(msb|datatemp);};
   		PORTA&=~128;
    } while (!interruptflag&&(page_counter<length));                  // loop while button for recording (button 1) is pressed
    //zapni preruseni
	sei();
    SPCR = 0x00;                            // disable SPI        
}


void RekniSlova()
{
	  //vystup libovolneho cisla v parametru
	  //format NF cislo nebo od Metexu
      unsigned char k=3;unsigned long lcislo;
	  unsigned char zacarkou=0;unsigned char bylohm=0;
	  while (k<CmdPos&&k<18&&!interruptflag) {	   
	   if (isdigit(CmdBuf[k])) {
	    //ciselny udaj
		   lcislo=0;
		   while (isdigit(CmdBuf[k])&&k<18&&(k<CmdPos)) {
		    lcislo*=10;
			lcislo+=CmdBuf[k]-'0';
			if (lcislo==0&&zacarkou) {
			 RekniSlovo(_NULA);
			}
			k++;
		   }	  	  
		   if (lcislo||!zacarkou) {RekniCislo(lcislo);}
	   } else {	   
	    //pismeno
		
		if (CmdBuf[k]=='-') { RekniSlovo(_MINUS);}
		if (CmdBuf[k]==','||CmdBuf[k]=='.') { RekniSlovo(_CELA);zacarkou=1;}
		
		if (!metex||k>5) {
			if (CmdBuf[k]=='p') { RekniSlovo(_PIKO);}
			if (CmdBuf[k]=='n') { RekniSlovo(_NANO);}
			if (CmdBuf[k]=='m'&&!bylohm) { RekniSlovo(_MILI);} //nerikej kdyz je text "Ohm" z metexu
			if (CmdBuf[k]=='M') { RekniSlovo(_MEGA);}
			if (CmdBuf[k]=='k') { RekniSlovo(_KILO);}
			if (CmdBuf[k]=='G') { RekniSlovo(_GIGA);}
		
			if (CmdBuf[k]=='O') { RekniSlovo(_OHMU);bylohm=1;}
			if (CmdBuf[k]=='V') { RekniSlovo(_VOLT);}
			if (CmdBuf[k]=='A') { RekniSlovo(_AMPER);}
			if (CmdBuf[k]=='P') { RekniSlovo(_PASCAL);}
			if (CmdBuf[k]=='E') { RekniSlovo(_METR);}
			if (CmdBuf[k]=='g') { RekniSlovo(_GRAM);}		
		}		
 	    k++;
	  }
    }
} 

void Recognize(void) 
{
		   fprintf_P(seriak,PSTR("__NACTENI PRIKAZU: Nacitam hlasovou informaci.\n"));
		   unsigned char i=LoadVoice(),vitez;
		   unsigned short es,min;
		   if (!i) {
			fprintf_P(seriak,PSTR("_Zpracovavam hlasovou informaci.\n"));
			FitVoice();
			//otestuj se vsemi sablonami z eeprom
			vitez=0;
			min=65000;
			if (CmdBuf[1]=='R') {
			 for (i=0;i<16;i++) {
			  LoadPat(i);
			  es=ErrScore();
			  if (es<min) {
			   min=es;
			   vitez=i;
			  }
			  fprintf_P(seriak,PSTR("_CHYBOVE SKORE PRO SABLONU %3u:%u\n"),i,es);
			 } 
			} else {
			 vitez=0;
			 for (i=0;i<16;i++) {
			  if (CmdBuf[(unsigned char)i+3]=='1') {
			   LoadPat(i);
			   es=ErrScore();
			   LoadPat(i+8);
			   es+=ErrScore();
			   if (es<min) {
			    min=es;
			    vitez=i;
			   }
			   fprintf_P(seriak,PSTR("_CHYBOVE SKORE PRO SABLONU %3u:%u\n"),i,es);
			  }; 
			 } 
			}	
			if (conf&LEDSPEAK)
			 {SpkLedOn();DelayMs(250);SpkLedOff();}
			fprintf_P(seriak,PSTR("OK %u %u\n"),vitez,min);	
            
		   } else {
			if (conf&LEDSPEAK&&!interruptflag)
			 {LedOn();DelayMs(250);}
		    fprintf_P(seriak,PSTR("ER %u\n"),i);
           }	
};




int main(void) {
 unsigned char i; //pocitadlo
// unsigned char stav=0,prikaz;
 //INICIALIZACE
 //porty
 setup();
 LedOn();
 PwrOff(); //vypni DA prevodnik a zesilovac
 //dame 100ms na ustaleni napeti...
 DelayMs(100); 
 // Timer/Counter 1 initialization
 // Clock source: System Clock
 // Clock value: 16000,000 kHz
 // Mode: Normal top=FFFFh
 // OC1A output: Discon.
 // OC1B output: Discon.
 // Noise Canceler: Off
 // Input Capture on Falling Edge
 TCCR1A=0x00;
 TCCR1B=0x01;
 TCNT1=0x00;
 //AD prevodnik
 a2dInit();
 //zvolime napred interni vstup
 a2dSetChannel(0);
 a2dStartConvert(); //jedna prazdna konverze aby se inicializoval prevodnik
 
 if ((PIND&4)==0) {
  //METEX rezim
  uart_init(832);
  metex=1;
  UCSRC=128+4;
 } else {
  //normalni rezim
  //seriovy kanal 
  uart_init(51);
  metex=0;
  
 }
 seriak=fdevopen(uart_transmit, NULL, 0);
 //vyprazdnime buffer seriaku
 while (UCSRA & (1<<RXC)) {ReceiveByte();}
 //zapni preruseni (enablovano je od USARTu)
 sei();
 
 fprintf_P(seriak,PSTR("_Pripraven v."VERZE"\n"));  
 LedOff(); 
 unsigned char k;unsigned long lcislo;
 CommandDone();
 CommandDone();
 format=WAVE;
 while(1)
 {
 if (!(PIND&8)&&metex) {
  //stisk tlacitka v METEX rezimu odesle 'A' do METEXu
  uart_transmit('A');
  DelayMs(25);
 }
 //je-li stisknuto tlacitko, posli informaci "REQUEST" (mimo METEX rezim)
 if (!(PIND&8)&&((conf&VRBUTTON)>0)&&!metex) {
  fprintf_P(seriak,PSTR("REQUEST\n"));
  DelayMs(25);
 }

 if (CmdFlag) {
   //zapni LED
   if (conf&LEDBUSY) {
    LedOn();
   };
   //je-li aktivni METEX rezim
   if (!(PIND&4)) {
    if (CmdPos==13) {
	 //pokud mame jiny pocet znaku, nemame cely datagram od metexu
	 //precteme datagram
	 RekniSlova();
	};
   } else 
   {
	   //normalni vykonani prikazu
	   uart_transmit('_');
	   for (i=0;i<CmdPos;i++) {uart_transmit(CmdBuf[i]);}
	   fprintf_P(seriak,PSTR(" (%d znaku)\n"),CmdPos);
	   interruptflag=0;
	   //prikazy 
	   if (CmdPos>=2) {
		if (CmdBuf[0]=='I') {
		 //prepinani vstupu pro nahravani		 
		 if (CmdBuf[1]=='E') {		  
		  a2dSetChannel(1);
		  fprintf_P(seriak,PSTR("_INPUT EXT (1)\n"));
		 }
		 if (CmdBuf[1]=='I') {		  
		  a2dSetChannel(0);
		  fprintf_P(seriak,PSTR("_INPUT INT (0)\n"));
		 }
         a2dStartConvert(); //jedna prazdna konverze aby se inicializoval prevodnik 		 
		} 
		if (CmdBuf[0]=='C') {
		 //nastaveni konfiguracniho slova
		 if (CmdBuf[1]=='S') {
		  DecodeLF();
		  conf=(unsigned char)from;
		  fprintf_P(seriak,PSTR("OK %d\n"),conf);
		  PwrOn();
		 }		 
		 //nastaveni diod
		 if (CmdBuf[1]=='L') {
		  DecodeLF();
		  PORTD&=0x0F;
		  PORTD|=((unsigned char)from<<4);
		  fprintf_P(seriak,PSTR("OK %d\n"),conf);
		 }
		} 
		if (CmdBuf[0]=='F') {
		 //format zvuku	 
		 if (CmdBuf[1]=='W') {
		  format=WAVE;
		  fprintf_P(seriak,PSTR("_WAVE 8\n"));
		 }
		 if (CmdBuf[1]=='A') {
		  format=ADPCM;
		  fprintf_P(seriak,PSTR("_ADPCM 16\n"));
		 }
		 if (CmdBuf[1]=='P') {
		  format=ADPCM8;
		  fprintf_P(seriak,PSTR("_ADPCM 8\n"));
		 }
		 if (CmdBuf[1]=='F') {
		  format=WAVE16;
		  fprintf_P(seriak,PSTR("_WAVE 16\n"));
		 }
		 fprintf_P(seriak,PSTR("OK %d\n"),format);
		} 
		if (CmdBuf[0]=='V') {
		 //nastaveni hlasitosti
		 if (CmdBuf[1]=='S') {
		  DecodeLF();
		  vole=(unsigned char)from;
		  vori=(unsigned char)length;
		  fprintf_P(seriak,PSTR("OK %d %d\n"),vole,vori);
		 }
		}
		if (CmdBuf[0]=='W') {
		 //prikazy pro praci s hlasovym zaznamnikem
		 PwrOn();
		 if (CmdBuf[1]=='P') {
		  //prehravani zvuku
		  //format WP odsektoru pocetsektoru
		  DecodeLF();
		  //mame 2 udaje length, from
		  fprintf_P(seriak,PSTR("_Prehravam od %d, delka %d\n"),from,length);
		  playback(from,length);
		  fprintf_P(seriak,PSTR("OK %d\n"),page_counter);
		 }
		 if (CmdBuf[1]=='R') {
		  //prehravani zvuku
		  //format WR odsektoru pocetsektoru
		  DecodeLF();
		  //mame 2 udaje length, from
		  fprintf_P(seriak,PSTR("_Nahravam od %d, delka %d\n"),from,length);
		  recording(from,length);
		  fprintf_P(seriak,PSTR("OK %d\n"),page_counter);
		 }
		 if (CmdBuf[1]=='E') {
		  //mazani flash
		  //format WE odbloku dobloku
		  DecodeLF();
		  //mame 2 udaje length, from
		  fprintf_P(seriak,PSTR("_Mazu od %d po %d\n"),from,length);
		  EraseFlash(from,length);
		  fprintf_P(seriak,PSTR("OK\n"));
		 }	 
		 if (CmdBuf[1]=='L') {
		  //nacitani zvuku
		  //format WL odsektoru pocetsektoru
		  DecodeLF();
		  //mame 2 udaje length, from
		  fprintf_P(seriak,PSTR("_Cekam na data od %d, delka %d\n"),from,length);
		  loading(from,length);
		  fprintf_P(seriak,PSTR("OK %d\n"),page_counter);
		 }
		}
		if (CmdBuf[0]=='R') {
		 if (CmdBuf[1]=='W') {
		  k=3;
		  lcislo=0;
		  while (isdigit(CmdBuf[k])&&k<18) {
		   lcislo*=10;
		   lcislo+=CmdBuf[k]-'0';
		   k++;
		  }	 
		  if (lcislo>15) {lcislo=15;}
		  fprintf_P(seriak,PSTR("_NACTENI SABLONY %u: Nacitam hlasovou informaci.\n"),(unsigned char)lcislo);
		  i=LoadVoice();
		  if (!i) {
			fprintf_P(seriak,PSTR("_Zpracovavam hlasovou informaci.\n"));
			FitVoice();
			pattern=aktvoice;
			SavePat((unsigned char)lcislo);
			fprintf_P(seriak,PSTR("OK\n"));
		  } else {
			fprintf_P(seriak,PSTR("ER %u\n"),i);
		  }   
		 }
		 if (CmdBuf[1]=='R'||CmdBuf[1]=='S') {
		  Recognize();
		 }
		}
		if (CmdBuf[0]=='N') {
		 PwrOn();
		 //prikazy hlasoveho vystupu
		 //prepneme si format na WAVE ve funkci RekniSlovo;
		 if (CmdBuf[1]=='S') {
		  //vystup cisla v parametru
		  //format NS cislo 
		  DecodeLF();
		  //mame 2 udaje length, from
		  fprintf_P(seriak,PSTR("_Rikam od %d, delka %d\n"),from,length);
		  RekniSlovo((unsigned char)from);
		  fprintf_P(seriak,PSTR("OK %d\n"),page_counter);
		 }
		 if (CmdBuf[1]=='I') {
		  //vystup cisla v parametru
		  //format NI cislo 
		  k=3;
		  lcislo=0;
		  while (isdigit(CmdBuf[k])&&k<18) {
		   lcislo*=10;
		   lcislo+=CmdBuf[k]-'0';
		   k++;
		  }	  	  
		  fprintf_P(seriak,PSTR("_Rikam od %lu\n"),lcislo);
		  RekniCislo(lcislo);
		  fprintf_P(seriak,PSTR("OK %d\n"),page_counter);
		 }
		 if (CmdBuf[1]=='F') {
			RekniSlova();
			fprintf_P(seriak,PSTR("OK %d\n"),page_counter);
		 }		 
		} 
	   }
	}	
   CommandDone();
   if (conf&LEDBUSY) {
    LedOff();
   };
   if (conf&PWRSAVE) {
    PwrOff();
   };
  }
  DelayMs(1);
 }
}
