/*
 Obsahuje funkce pro jednoduche rozpoznavani reci
*/

//konstaty chybovych kodu

#define	LV_OK	0
#define	LV_SILENCE	1
#define LV_TOOLONG 2
#define LV_TOOSHORT 3
#define LV_INTERRUPTED 4

//struktura, kam se ukladaji prave nactena data

struct TVoicePattern {
	unsigned char hi[128];
	unsigned char lo[128];	
	unsigned short length;
} vopat;

//struktura, kam se uklada prevzorkovane spektrum signalu

struct TFittedVoice {
	unsigned char hi[16];
	unsigned char lo[16];	
	//pozor:v bajtu hi[15] je ulozena delka ve vzorcich pro porovnani delek zvuku
} pattern,aktvoice;

//ulozi do eeprom pattern na pozici p
void SavePat(unsigned char p)
{
	unsigned char i;
    fprintf_P(seriak,PSTR("__SavePat ... start\n"));   
	//uloz [lo]
	for (i=0;i<16;i++) {
	 eeprom_wb((unsigned short)p*32+i,pattern.lo[i]);
	}
	//uloz [hi]
	for (i=0;i<16;i++) {
	 eeprom_wb((unsigned short)p*32+i+16,pattern.hi[i]);
	}	
    fprintf_P(seriak,PSTR("__SavePat ... konec\n"));   
}

//nacte vzorek cislo p z EEPROM a ulozi ho do pattern
void LoadPat(unsigned char p)
{
	unsigned char i;
    //fprintf_P(seriak,PSTR("__LoadPat ... start\n"));   
	//uloz [lo]
	for (i=0;i<16;i++) {
	 pattern.lo[i]=eeprom_rb((unsigned short)p*32+i);	 
	}
	//uloz [hi]
	for (i=0;i<16;i++) {
	 pattern.hi[i]=eeprom_rb((unsigned short)p*32+i+16);	 
	}	
    //fprintf_P(seriak,PSTR("__LoadPat ... konec\n"));   
}


//vrati absolutni hodnotu rozdilu 2 cisel
unsigned char Delta(unsigned char a1, unsigned char b1)
{
 if (a1>b1) {
  return a1-b1;
 } else {
  return b1-a1;
 }
}
//vrati euklidovskou vzdalenost 2 cisel
unsigned char Delta2(unsigned char a1, unsigned char b1)
{
 if (a1>b1) {
  return (a1-b1)*(a1-b1);
 } else {
  return (b1-a1)*(b1-a1);
 }
}

//vypocita chybove skore mezi aktvoice a vopat
unsigned short ErrScore() {
 unsigned short tmp=0;
 unsigned char i;
 //rozdil prubehu
 for (i=0;i<16;i++) {
  tmp+=3*Delta2(aktvoice.lo[i],pattern.lo[i]);
  if (i<15) {tmp+=Delta2(aktvoice.hi[i],pattern.hi[i]);}
  if (tmp>20000) {
   return 65000;
  }
 }
 //rozdil delky
 if (Delta(aktvoice.hi[15],pattern.hi[15])>5) {
  tmp+=20*Delta(aktvoice.hi[15],pattern.hi[15]);
 } 
 return tmp;
}


//prepocita zvuk z vopat a ulozi ho do aktvoice
void FitVoice(void) {
    unsigned char i,akt=0;
	unsigned char n;
	unsigned short hisum;unsigned short losum;
    fprintf_P(seriak,PSTR("__FitVoice ... start\n"));   
	if (vopat.length<17) {
	 for (i=0;i<16;i++) {
	  hisum=0;losum=0;n=0;
	  while ((unsigned short)akt<=((unsigned short)((unsigned short)i*(vopat.length+1)) / 16)) { 
	   losum+=vopat.lo[akt];
	   hisum+=vopat.hi[akt];
	   n++;
	   akt++;
	  }
	  akt--;
	  aktvoice.lo[i]=losum/n;
	  aktvoice.hi[i]=hisum/n;
	 }
	} else {
	 for (i=0;i<16;i++) {
	  hisum=0;losum=0;n=0;
      while ((unsigned short)akt<=((unsigned short)i*(vopat.length) / 15)) {
	   losum+=vopat.lo[akt];
	   hisum+=vopat.hi[akt];
	   n++;
	   akt++;
	  } 
	  aktvoice.lo[i]=losum/n;
	  aktvoice.hi[i]=hisum/n;
	 } 
    }
    aktvoice.hi[15]=vopat.length;

    fprintf_P(seriak,PSTR("__FitVoice vypis dat: [lo],[hi]\n_"));   
    for (i=0;i<16;i++) {
     fprintf_P(seriak,PSTR("%.3u,%.3u;"),aktvoice.lo[i],aktvoice.hi[i]);
    }
    fprintf_P(seriak,PSTR("\n"));
} 



//nacita zvuk ze vstupu do [vopat]
unsigned char LoadVoice(void) {
 unsigned short vzorku=0;
 unsigned char zacal=0,tichocnt=0,hi=0,lo=0,result; 
 unsigned char celkem=0,state;
 unsigned short deltacnt=0,adres,noisecomp=500;
 unsigned char laststate,li,lhi=0;
 unsigned char lastten[10];
 fprintf_P(seriak,PSTR("__LoadVoice ... start\n"));
 for (state=0;state<200;state++) {
  DelayUs(100);
  a2dStartConvert();
  adres=a2dConvert10bit();
  if (adres>noisecomp) {
   noisecomp=adres;
  }
  if (state>190) {lastten[state-190]=adres>>2;};
 }
 if (noisecomp>650) {
  noisecomp=650;
 }
 fprintf_P(seriak,PSTR("__LoadVoice ... kompenzace hluku = %u\n"),noisecomp);
 laststate=0;
 fprintf_P(seriak,PSTR("__LoadVoice ... mluvte\n"));
 TCNT1=0;
 if (conf&LEDSPEAK) {
  LedOff();
  SpkLedOn();
 };
 do {
  if (TCNT1>1450) {
   //vejdi sem kazdych 1/11025 sec
   TCNT1=0;
   //inkrementuj pocitadlo vzorku
   vzorku++;
   //nacti z AD prevodniku
   a2dStartConvert();
   //posun lastten ve volnem case
   for (li=1;li<10;li++) {
    lastten[li-1]=lastten[li];
   }
   //cekej na dokonceni prevodu a ziskej data
   loop_until_bit_is_set(ADCSR, ADIF);
   adres=a2dConvert10bit();
   lastten[9]=adres>>2;
   //detekuj zmenu vstupu
   state=(unsigned short)adres>noisecomp;
   if (state==laststate) {
    deltacnt++;
   } else {
    laststate=state;
	if (deltacnt>9) {
	 lo++;
	} else {
	 //hi++;
	}
	deltacnt=0;
   }
   if ((lastten[5]>(lastten[0]+2))&&(lastten[5]>(lastten[9]+2))&&(lhi==0)) {
    hi++;
	lhi=1;
   }
   if ((lastten[5]+2<(lastten[0]))&&(lastten[5]+2<(lastten[9]))&&lhi) {
    hi++;
	lhi=0;
   }
   if (vzorku%256==0) {
    if (hi>3&&lo>1) {
	 zacal=1;
	 vopat.hi[celkem]=hi;
	 vopat.lo[celkem]=lo;
	 celkem++;
	 if (conf&LEDSPEAK)
	  {LedOn();}
	} else { 
	 if (zacal) {
	  tichocnt++;
	 }
	 if (conf&LEDSPEAK)
	  {LedOff();}
	}
    hi=0;lo=0;
   }
  };
 } while ((unsigned short)vzorku<60000&&tichocnt<15&&celkem<126&&!interruptflag);
 if (conf&LEDSPEAK)
	  {LedOff();SpkLedOff();}

 fprintf_P(seriak,PSTR("__LoadVoice ... end vzorku: %u, tichocnt=%u\n"),vzorku,tichocnt);   
 fprintf_P(seriak,PSTR("__LoadVoice ... celkem OK vzorku: %u\n"),celkem);   
 result=LV_OK;
 if (celkem>=126||(unsigned short)vzorku>=60000) {
  result=LV_TOOLONG;
 }
 if (celkem<6) {
  result=LV_TOOSHORT;
 }
 if (interruptflag) {
  result=LV_INTERRUPTED;
 }
 if (celkem) {
  celkem--;
  vopat.length=celkem;
 /* for (zacal=0;zacal<=celkem;zacal++) {
   fprintf_P(seriak,PSTR("  %.3u,%.3u\n"),vopat.lo[zacal],vopat.hi[zacal]);
  }*/
 } 
 return result;
 
};

